@@ -262,11 +262,11 @@ void Titraqform::loadData(QTextStream &Tstream)
         }
     }
 
-    // Start sorting order and direction correctly according to user preferences
-    int nSortcol = (int)m_pPrefs->getNumber(TITRAQ_PREFSORTCOL, TITRAQ_DEFSORTCOL);
-    bool bSortdir = m_pPrefs->getBool(TITRAQ_PREFSORTDIR, TITRAQ_DEFSORTDIR);
-    m_pMaintable->setSortdir(!bSortdir); // Hack! Fake sortdir so we start right
-    m_pMaintable->sortColumn(nSortcol, bSortdir);
+//    // Start sorting order and direction correctly according to user preferences
+//    int nSortcol = (int)m_pPrefs->getNumber(TITRAQ_PREFSORTCOL, TITRAQ_DEFSORTCOL);
+//    bool bSortdir = m_pPrefs->getBool(TITRAQ_PREFSORTDIR, TITRAQ_DEFSORTDIR);
+//    m_pMaintable->setSortdir(!bSortdir); // Hack! Fake sortdir so we start right
+//    m_pMaintable->sortColumn(nSortcol, bSortdir);
 
     // Write nonsaving line numbers for all rows
     for (int nIter = m_pMaintable->numRows() - 1; nIter >= 0; nIter--)
@@ -436,9 +436,9 @@ void Titraqform::setRowdata(QString &Rowdata) const
         Remark = Rowstream.readLine();                          // Remark is a whole line
 
         // Set the table row data one field at a time, skipping seps inbetween
+        // Importantly, do not copy over the GUID, which must be unique each row
         m_pMaintable->setText(nCurrentrow + nIter, TITRAQ_IDXLINE, Line);
         m_pMaintable->setText(nCurrentrow + nIter, TITRAQ_IDXUSER, User);
-        m_pMaintable->setText(nCurrentrow + nIter, TITRAQ_IDXGUID, Guid);
         m_pMaintable->setText(nCurrentrow + nIter, TITRAQ_IDXCRC, Crc);
         m_pMaintable->setText(nCurrentrow + nIter, TITRAQ_IDXREV, Rev);
         m_pMaintable->setText(nCurrentrow + nIter, TITRAQ_IDXDATE, Date);
